/*
   JWindowsScheduler - Java access library for the windows task scheduler
   Copyright (C) 2002 Steffen Siebert <siebert@SteffenSiebert.de>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <windows.h>
#include <initguid.h>
#include <ole2.h>
#include <mstask.h>
#include <msterr.h>
#include <wchar.h>

#include "ole.h"

#define BEGIN_YEAR 0
#define BEGIN_MONTH 1
#define BEGIN_DAY 2
#define START_HOUR 3
#define START_MINUTE 4
#define END_YEAR 5
#define END_MONTH 6
#define END_DAY 7
#define MINUTES_DURATION 8
#define MINUTES_INTERVAL 9
#define RGFLAGS 10
#define TRIGGER_TYPE 11
#define DAYS_INTERVAL 12
#define WEEKS_INTERVAL 13
#define RGF_DAYS_OF_WEEK 14
#define RGF_DAYS 15
#define RGF_MONTHS 16
#define WHICH_WEEK 17

/*
 * Creates an instance of the given exception class and throws it.
 */
void JNU_ThrowByName(JNIEnv *env, const char *className, const char *msg, const HRESULT hr) {
  jclass cls = env->FindClass(className);

  if (cls != NULL) {
    char buffer[512];
    _snprintf(buffer, 512, msg, hr);
    buffer[511] = '\0';
    env->ThrowNew(cls, buffer);
    env->DeleteLocalRef(cls);
  }
}

JNIEXPORT jint JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_CoInitialize(JNIEnv *env, jclass) {
  return CoInitialize(NULL);
}

JNIEXPORT void JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_CoUninitialize(JNIEnv *env, jclass) {
  CoUninitialize();
}

JNIEXPORT jint JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_CoCreateInstance(JNIEnv *env, jclass) {
  HRESULT hr = ERROR_SUCCESS;
  ITaskScheduler *pITS;
  hr = CoCreateInstance(CLSID_CTaskScheduler,
                        NULL,
                        CLSCTX_INPROC_SERVER,
                        IID_ITaskScheduler,
                        (void **) &pITS);
  if (FAILED(hr)) {
    JNU_ThrowByName(env, "de/steffensiebert/jwindowsscheduler/OleException", "CoCreateInstance failed with returncode %i", HRESULT_FROM_WIN32(hr));
    return NULL;
  } else {
    return (jint) pITS;
  }
}

JNIEXPORT void JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_ReleaseTaskScheduler(JNIEnv *env, jclass, jint taskScheduler) {
  ITaskScheduler *pITS = (ITaskScheduler *)taskScheduler;
  pITS->Release();
}

JNIEXPORT jint JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_Activate(JNIEnv *env, jclass, jint taskScheduler, jstring taskName) {
  HRESULT hr = ERROR_SUCCESS;
  ITaskScheduler *pITS = (ITaskScheduler *)taskScheduler;
  ITask *pITask;
  LPCWSTR lpcwszTaskName = env->GetStringChars(taskName, NULL);
  hr = pITS->Activate(lpcwszTaskName,
                      IID_ITask,
                      (IUnknown**) &pITask);
  if (FAILED(hr)) {
    JNU_ThrowByName(env, "de/steffensiebert/jwindowsscheduler/OleException", "Activate failed with returncode %i", hr);
    return NULL;
  } else {
    return (jint) pITask;
  }
}

JNIEXPORT jint JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_CreateTrigger(JNIEnv *env, jclass, jint task) {
  HRESULT hr = ERROR_SUCCESS;
  ITask *pITask = (ITask *)task;
  ITaskTrigger *pITaskTrigger;
  WORD piNewTrigger;
  hr = pITask->CreateTrigger(&piNewTrigger,
                             &pITaskTrigger);
  if (FAILED(hr)) {
    JNU_ThrowByName(env, "de/steffensiebert/jwindowsscheduler/OleException", "CreateTrigger failed with returncode %i", hr);
    return NULL;
  } else {
    return (jint) pITaskTrigger;
  }
}

JNIEXPORT jint JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_NewWorkItem(JNIEnv *env, jclass, jint taskScheduler, jstring taskName) {
  HRESULT hr = ERROR_SUCCESS;
  ITaskScheduler *pITS = (ITaskScheduler *)taskScheduler;
  ITask *pITask;
  LPCWSTR pwszTaskName = env->GetStringChars(taskName, NULL);
  hr = pITS->NewWorkItem(pwszTaskName,           // Name of task
                         CLSID_CTask,            // Class identifier 
                         IID_ITask,              // Interface identifier
                         (IUnknown**)&pITask); // Address of task interface
  if (FAILED(hr)) {
    if (hr == HRESULT_FROM_WIN32(ERROR_FILE_EXISTS)) {
      JNU_ThrowByName(env, "de/steffensiebert/jwindowsscheduler/OleTaskExistException", "NewWorkItem failed, task with same name exists!", hr);
    } else {
      JNU_ThrowByName(env, "de/steffensiebert/jwindowsscheduler/OleException", "NewWorkItem failed with returncode %i", hr);
    }
    return NULL;
  } else {
    return (jint) pITask;
  }
}

JNIEXPORT jint JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_QueryInterface(JNIEnv *env, jclass, jint task) {
  HRESULT hr = ERROR_SUCCESS;
  ITask *pITask = (ITask *)task;
  IPersistFile *pIPersistFile;
  hr = pITask->QueryInterface(IID_IPersistFile,
                              (void **)&pIPersistFile);
  if (FAILED(hr)) {
    JNU_ThrowByName(env, "de/steffensiebert/jwindowsscheduler/OleException", "QueryInterface failed with returncode %i", hr);
    return NULL;
  } else {
    return (jint) pIPersistFile;
  }
}


JNIEXPORT void JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_Save(JNIEnv *env, jclass, jint persistFile) {
  HRESULT hr = ERROR_SUCCESS;
  IPersistFile *pIPersistFile = (IPersistFile *)persistFile;
  hr = pIPersistFile->Save(NULL,
                           TRUE);
  pIPersistFile->Release();
  if (FAILED(hr)) {
    JNU_ThrowByName(env, "de/steffensiebert/jwindowsscheduler/OleException", "Save failed with returncode %i", hr);
    return;
  }
}
  
JNIEXPORT void JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_ReleaseTask(JNIEnv *env, jclass, jint task) {
  ITask *pITask = (ITask *)task;
  pITask->Release();
}

JNIEXPORT jint JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_GetTriggerCount(JNIEnv *env, jclass, jint task) {
  HRESULT hr = ERROR_SUCCESS;
  WORD plTriggerCount;
  ITask *pITask = (ITask *)task;
  hr = pITask->GetTriggerCount (&plTriggerCount);
  if (FAILED(hr)) {
    JNU_ThrowByName(env, "de/steffensiebert/jwindowsscheduler/OleException", "GetTriggerCount failed with returncode %i", hr);
    return NULL;
  } else {
    return (jint) plTriggerCount;
  }
}

JNIEXPORT jstring JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_GetTriggerString(JNIEnv *env, jclass, jint task, jint trigger) {
  ITask *pITask = (ITask *)task;
  LPWSTR ppwszTrigger;
  pITask->GetTriggerString (trigger,
                            &ppwszTrigger); 
  jstring str = env->NewString(ppwszTrigger, wcslen(ppwszTrigger));
  return str;
}

// JNIEXPORT jint JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_GetTrigger(JNIEnv *env, jclass, jint task, jint trigger) {
//   HRESULT hr = ERROR_SUCCESS;
//   IEnumWorkItems *pIEnum;
//   ITask *pIT = (ITask *)task;
//   hr = pIT->GetTask(&pIEnum);
//   if (FAILED(hr)) {
//     // FIXME Exception
//     wprintf(L"Failed calling ITaskScheduler::Enum: ");
//     wprintf(L"error = 0x%x\n",hr);
//     return NULL;
//   } else {
//     return (jint) pIEnum;
//   }
// }

JNIEXPORT jint JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_GetTriggerType(JNIEnv *env, jclass, jint task, jint trigger) {
  ITask *pITask = (ITask *)task;
  ITaskTrigger *pITaskTrigger;
  TASK_TRIGGER pTrigger;
  ZeroMemory(&pTrigger, sizeof(TASK_TRIGGER));
  pTrigger.cbTriggerSize = sizeof(TASK_TRIGGER); 
  pITask->GetTrigger(trigger,
                     &pITaskTrigger); 
  pITaskTrigger->GetTrigger(&pTrigger);
  return (jint) pTrigger.TriggerType;
}

// JNIEXPORT jint JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_GetTriggerBeginYear(JNIEnv *env, jclass, jint task, jint trigger) {
//   ITask *pITask = (ITask *)task;
//   ITaskTrigger *pITaskTrigger;
//   TASK_TRIGGER pTrigger;
//   ZeroMemory(&pTrigger, sizeof(TASK_TRIGGER));
//   pTrigger.cbTriggerSize = sizeof(TASK_TRIGGER); 
//   pITask->GetTrigger(trigger,
//                      &pITaskTrigger); 
//   pITaskTrigger->GetTrigger(&pTrigger);
//   return (jint) pTrigger.wBeginYear;
// }

// JNIEXPORT jint JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_GetTriggerBeginMonth(JNIEnv *env, jclass, jint task, jint trigger) {
//   ITask *pITask = (ITask *)task;
//   ITaskTrigger *pITaskTrigger;
//   TASK_TRIGGER pTrigger;
//   ZeroMemory(&pTrigger, sizeof(TASK_TRIGGER));
//   pTrigger.cbTriggerSize = sizeof(TASK_TRIGGER); 
//   pITask->GetTrigger(trigger,
//                      &pITaskTrigger); 
//   pITaskTrigger->GetTrigger(&pTrigger);
//   return (jint) pTrigger.wBeginMonth;
// }

JNIEXPORT jint JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_GetTriggerValues(JNIEnv *env, jclass, jint task, jint trigger, jintArray array) {
  ITask *pITask = (ITask *)task;
  ITaskTrigger *pITaskTrigger;
  TASK_TRIGGER pTrigger;
  ZeroMemory(&pTrigger, sizeof(TASK_TRIGGER));
  pTrigger.cbTriggerSize = sizeof(TASK_TRIGGER); 
  pITask->GetTrigger(trigger,
                     &pITaskTrigger); 
  pITaskTrigger->GetTrigger(&pTrigger);
  jint value;
  value = pTrigger.wBeginYear;
  env->SetIntArrayRegion(array,BEGIN_YEAR,1,&value);
  value = pTrigger.wBeginMonth;
  env->SetIntArrayRegion(array,BEGIN_MONTH,1,&value);
  value = pTrigger.wBeginDay;
  env->SetIntArrayRegion(array,BEGIN_DAY,1,&value);
  value = pTrigger.wStartHour;
  env->SetIntArrayRegion(array,START_HOUR,1,&value);
  value = pTrigger.wStartMinute;
  env->SetIntArrayRegion(array,START_MINUTE,1,&value);
  value = pTrigger.wEndYear;
  env->SetIntArrayRegion(array,END_YEAR,1,&value);
  value = pTrigger.wEndMonth;             
  env->SetIntArrayRegion(array,END_MONTH,1,&value);
  value = pTrigger.wEndDay;               
  env->SetIntArrayRegion(array,END_DAY,1,&value);
  value = pTrigger.MinutesDuration;
  env->SetIntArrayRegion(array,MINUTES_DURATION,1,&value);
  value = pTrigger.MinutesInterval;      
  env->SetIntArrayRegion(array,MINUTES_INTERVAL,1,&value);
  value = pTrigger.rgFlags;              
  env->SetIntArrayRegion(array,RGFLAGS,1,&value);
  value = pTrigger.TriggerType;
  env->SetIntArrayRegion(array,TRIGGER_TYPE,1,&value);
  if (pTrigger.TriggerType == TASK_TIME_TRIGGER_DAILY) {
    value = pTrigger.Type.Daily.DaysInterval;
    env->SetIntArrayRegion(array,DAYS_INTERVAL,1,&value);
  } else if (pTrigger.TriggerType == TASK_TIME_TRIGGER_WEEKLY) {
    value = pTrigger.Type.Weekly.WeeksInterval;
    env->SetIntArrayRegion(array,WEEKS_INTERVAL,1,&value);
    value = pTrigger.Type.Weekly.rgfDaysOfTheWeek;
    env->SetIntArrayRegion(array,RGF_DAYS_OF_WEEK,1,&value);
  } else if (pTrigger.TriggerType == TASK_TIME_TRIGGER_MONTHLYDATE) {
    value = pTrigger.Type.MonthlyDate.rgfDays;
    env->SetIntArrayRegion(array,RGF_DAYS,1,&value);
    value = pTrigger.Type.MonthlyDate.rgfMonths;
    env->SetIntArrayRegion(array,RGF_MONTHS,1,&value);
  } else if (pTrigger.TriggerType == TASK_TIME_TRIGGER_MONTHLYDOW) {
    value = pTrigger.Type.MonthlyDOW.wWhichWeek;
    env->SetIntArrayRegion(array,WHICH_WEEK,1,&value);
    value = pTrigger.Type.MonthlyDOW.rgfDaysOfTheWeek;
    env->SetIntArrayRegion(array,RGF_DAYS_OF_WEEK,1,&value);
    value = pTrigger.Type.MonthlyDOW.rgfMonths;
    env->SetIntArrayRegion(array,RGF_MONTHS,1,&value);
  }
  return (jint) 0;
}

JNIEXPORT void JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_SetTrigger(JNIEnv *env, jclass, jint trigger, jintArray array) {
  HRESULT hr = ERROR_SUCCESS;
  TASK_TRIGGER pTrigger;
  ZeroMemory(&pTrigger, sizeof(TASK_TRIGGER));
  pTrigger.cbTriggerSize = sizeof(TASK_TRIGGER); 
  jint value;
  env->GetIntArrayRegion(array,BEGIN_YEAR,1,&value);
  pTrigger.wBeginYear = value;
  env->GetIntArrayRegion(array,BEGIN_MONTH,1,&value);
  pTrigger.wBeginMonth = value;
  env->GetIntArrayRegion(array,BEGIN_DAY,1,&value);
  pTrigger.wBeginDay = value;
  env->GetIntArrayRegion(array,START_HOUR,1,&value);
  pTrigger.wStartHour = value;
  env->GetIntArrayRegion(array,START_MINUTE,1,&value);
  pTrigger.wStartMinute = value;
  env->GetIntArrayRegion(array,END_YEAR,1,&value);
  pTrigger.wEndYear = value;
  env->GetIntArrayRegion(array,END_MONTH,1,&value);
  pTrigger.wEndMonth = value;             
  env->GetIntArrayRegion(array,END_DAY,1,&value);
  pTrigger.wEndDay = value;               
  env->GetIntArrayRegion(array,MINUTES_DURATION,1,&value);
  pTrigger.MinutesDuration = value;
  env->GetIntArrayRegion(array,MINUTES_INTERVAL,1,&value);
  pTrigger.MinutesInterval = value;      
  env->GetIntArrayRegion(array,RGFLAGS,1,&value);
  pTrigger.rgFlags = value;              
  env->GetIntArrayRegion(array,TRIGGER_TYPE,1,&value);
  pTrigger.TriggerType = (TASK_TRIGGER_TYPE) value;
  if (pTrigger.TriggerType == TASK_TIME_TRIGGER_DAILY) {
    env->GetIntArrayRegion(array,DAYS_INTERVAL,1,&value);
    pTrigger.Type.Daily.DaysInterval = value;
  } else if (pTrigger.TriggerType == TASK_TIME_TRIGGER_WEEKLY) {
    env->GetIntArrayRegion(array,WEEKS_INTERVAL,1,&value);
    pTrigger.Type.Weekly.WeeksInterval = value;
    env->GetIntArrayRegion(array,RGF_DAYS_OF_WEEK,1,&value);
    pTrigger.Type.Weekly.rgfDaysOfTheWeek = value;
  } else if (pTrigger.TriggerType == TASK_TIME_TRIGGER_MONTHLYDATE) {
    env->GetIntArrayRegion(array,RGF_DAYS,1,&value);
    pTrigger.Type.MonthlyDate.rgfDays = value;
    env->GetIntArrayRegion(array,RGF_MONTHS,1,&value);
    pTrigger.Type.MonthlyDate.rgfMonths = value;
  } else if (pTrigger.TriggerType == TASK_TIME_TRIGGER_MONTHLYDOW) {
    env->GetIntArrayRegion(array,WHICH_WEEK,1,&value);
    pTrigger.Type.MonthlyDOW.wWhichWeek = value;
    env->GetIntArrayRegion(array,RGF_DAYS_OF_WEEK,1,&value);
    pTrigger.Type.MonthlyDOW.rgfDaysOfTheWeek = value;
    env->GetIntArrayRegion(array,RGF_MONTHS,1,&value);
    pTrigger.Type.MonthlyDOW.rgfMonths = value;
  }
  ITaskTrigger *pITaskTrigger = (ITaskTrigger *) trigger;
  hr = pITaskTrigger->SetTrigger (&pTrigger);
  if (FAILED(hr)) {
    JNU_ThrowByName(env, "de/steffensiebert/jwindowsscheduler/OleException", "SetTrigger failed with returncode %i", hr);
  }
  return;
}

JNIEXPORT jstring JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_GetApplicationName(JNIEnv *env, jclass, jint task) {
  HRESULT hr = ERROR_SUCCESS;
  LPWSTR lpwszApplicationName;
  ITask *pITask = (ITask *)task;
  hr = pITask->GetApplicationName(&lpwszApplicationName);
  if (FAILED(hr)) {
    JNU_ThrowByName(env, "de/steffensiebert/jwindowsscheduler/OleException", "GetApplicationName failed with returncode %i", hr);
    return NULL;
  } else {
    jstring str = env->NewString(lpwszApplicationName, wcslen(lpwszApplicationName));
    CoTaskMemFree(lpwszApplicationName);
    return str;
  }
}

JNIEXPORT void JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_SetApplicationName(JNIEnv *env, jclass, jint task, jstring applicationName) {
  HRESULT hr = ERROR_SUCCESS;
  ITask *pITask = (ITask *)task;
  LPCWSTR lpwszApplicationName = env->GetStringChars(applicationName, NULL);
  hr = pITask->SetApplicationName(lpwszApplicationName);
  if (FAILED(hr)) {
    JNU_ThrowByName(env, "de/steffensiebert/jwindowsscheduler/OleException", "SetApplicationName failed with returncode %i", hr);
    return;
  }
}

JNIEXPORT jstring JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_GetParameters(JNIEnv *env, jclass, jint task) {
  HRESULT hr = ERROR_SUCCESS;
  LPWSTR lpwszParameters;
  ITask *pITask = (ITask *)task;
  hr = pITask->GetParameters(&lpwszParameters);
  if (FAILED(hr)) {
    JNU_ThrowByName(env, "de/steffensiebert/jwindowsscheduler/OleException", "GetParameters failed with returncode %i", hr);
    return NULL;
  } else {
    jstring str = env->NewString(lpwszParameters, wcslen(lpwszParameters));
    CoTaskMemFree(lpwszParameters);
    return str;
  }
}

JNIEXPORT void JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_SetParameters(JNIEnv *env, jclass, jint task, jstring parameters) {
  HRESULT hr = ERROR_SUCCESS;
  ITask *pITask = (ITask *)task;
  LPCWSTR lpwszParameters = env->GetStringChars(parameters, NULL);
  hr = pITask->SetParameters(lpwszParameters);
  if (FAILED(hr)) {
    JNU_ThrowByName(env, "de/steffensiebert/jwindowsscheduler/OleException", "SetParameters failed with returncode %i", hr);
    return;
  }
}

JNIEXPORT jstring JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_GetWorkingDirectory(JNIEnv *env, jclass, jint task) {
  HRESULT hr = ERROR_SUCCESS;
  LPWSTR lpwszWorkingDirectory;
  ITask *pITask = (ITask *)task;
  hr = pITask->GetWorkingDirectory(&lpwszWorkingDirectory);
  if (FAILED(hr)) {
    JNU_ThrowByName(env, "de/steffensiebert/jwindowsscheduler/OleException", "GetWorkingDirectory failed with returncode %i", hr);
    return NULL;
  } else {
    jstring str = env->NewString(lpwszWorkingDirectory, wcslen(lpwszWorkingDirectory));
    CoTaskMemFree(lpwszWorkingDirectory);
    return str;
  }
}

JNIEXPORT void JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_SetWorkingDirectory(JNIEnv *env, jclass, jint task, jstring workingDirectory) {
  HRESULT hr = ERROR_SUCCESS;
  ITask *pITask = (ITask *)task;
  LPCWSTR lpwszWorkingDirectory = env->GetStringChars(workingDirectory, NULL);
  hr = pITask->SetWorkingDirectory(lpwszWorkingDirectory);
  if (FAILED(hr)) {
    JNU_ThrowByName(env, "de/steffensiebert/jwindowsscheduler/OleException", "SetWorkingDirectory failed with returncode %i", hr);
    return;
  }
}

JNIEXPORT jint JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_GetPriority(JNIEnv *env, jclass, jint task) {
  HRESULT hr = ERROR_SUCCESS;
  DWORD pdwPriority;
  ITask *pITask = (ITask *)task;
  hr = pITask->GetPriority(&pdwPriority);
  if (FAILED(hr)) {
    JNU_ThrowByName(env, "de/steffensiebert/jwindowsscheduler/OleException", "GetPriority failed with returncode %i", hr);
    return NULL;
  } else {
    return (jint) pdwPriority;
  }
}

JNIEXPORT void JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_SetPriority(JNIEnv *env, jclass, jint task, jint priority) {
  HRESULT hr = ERROR_SUCCESS;
  ITask *pITask = (ITask *)task;
  DWORD pdwPriority = (DWORD) priority;
  hr = pITask->SetPriority(pdwPriority);
  if (FAILED(hr)) {
    JNU_ThrowByName(env, "de/steffensiebert/jwindowsscheduler/OleException", "SetPriority failed with returncode %i", hr);
    return;
  }
}

JNIEXPORT jint JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_GetMaxRunTime(JNIEnv *env, jclass, jint task) {
  HRESULT hr = ERROR_SUCCESS;
  DWORD pdwRunTime;
  ITask *pITask = (ITask *)task;
  hr = pITask->GetMaxRunTime(&pdwRunTime);
  if (FAILED(hr)) {
    JNU_ThrowByName(env, "de/steffensiebert/jwindowsscheduler/OleException", "GetMaxRunTime failed with returncode %i", hr);
    return NULL;
  } else {
    return (jint) pdwRunTime;
  }
}

JNIEXPORT void JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_SetMaxRunTime(JNIEnv *env, jclass, jint task, jint maxRunTime) {
  HRESULT hr = ERROR_SUCCESS;
  ITask *pITask = (ITask *)task;
  DWORD pdwRunTime = (DWORD) maxRunTime;
  hr = pITask->SetMaxRunTime(pdwRunTime);
  if (FAILED(hr)) {
    JNU_ThrowByName(env, "de/steffensiebert/jwindowsscheduler/OleException", "SetMaxRunTime failed with returncode %i", hr);
    return;
  }
}

JNIEXPORT jstring JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_GetCreator(JNIEnv *env, jclass, jint task) {
  HRESULT hr = ERROR_SUCCESS;
  LPWSTR lpwszCreator;
  ITask *pITask = (ITask *)task;
  hr = pITask->GetCreator(&lpwszCreator);
  if (FAILED(hr)) {
    JNU_ThrowByName(env, "de/steffensiebert/jwindowsscheduler/OleException", "GetCreator failed with returncode %i", hr);
    return NULL;
  } else {
    jstring str = env->NewString(lpwszCreator, wcslen(lpwszCreator));
    CoTaskMemFree(lpwszCreator);
    return str;
  }
}

JNIEXPORT jint JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_GetFlags(JNIEnv *env, jclass, jint task) {
  HRESULT hr = ERROR_SUCCESS;
  DWORD pdwFlags;
  ITask *pITask = (ITask *)task;
  hr = pITask->GetFlags(&pdwFlags);
  if (FAILED(hr)) {
    JNU_ThrowByName(env, "de/steffensiebert/jwindowsscheduler/OleException", "GetFlags failed with returncode %i", hr);
    return NULL;
  } else {
    return (jint) pdwFlags;
  }
}

JNIEXPORT void JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_SetFlags(JNIEnv *env, jclass, jint task, jint flags) {
  HRESULT hr = ERROR_SUCCESS;
  ITask *pITask = (ITask *)task;
  DWORD pdwFlags = (DWORD) flags;
  hr = pITask->SetFlags(pdwFlags);
  if (FAILED(hr)) {
    JNU_ThrowByName(env, "de/steffensiebert/jwindowsscheduler/OleException", "SetFlags failed with returncode %i", hr);
    return;
  }
}

JNIEXPORT void JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_SetAccountInformation(JNIEnv *env, jclass, jint task, jstring accountName, jstring password) {
  HRESULT hr = ERROR_SUCCESS;
  ITask *pITask = (ITask *)task;
  LPCWSTR pwszAccountName = env->GetStringChars(accountName, NULL);
  LPCWSTR pwszPassword = NULL;
  if (password != NULL) {
    LPCWSTR pwszPassword = env->GetStringChars(password, NULL);
  }
  hr = pITask->SetAccountInformation(pwszAccountName,
                                     pwszPassword);
  if (FAILED(hr)) {
    JNU_ThrowByName(env, "de/steffensiebert/jwindowsscheduler/OleException", "SetAccountInformation failed with returncode %i", hr);
    return;
  }
}

JNIEXPORT jstring JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_GetComment(JNIEnv *env, jclass, jint task) {
  HRESULT hr = ERROR_SUCCESS;
  LPWSTR lpwszComment;
  ITask *pITask = (ITask *)task;
  hr = pITask->GetComment(&lpwszComment);
  if (FAILED(hr)) {
    JNU_ThrowByName(env, "de/steffensiebert/jwindowsscheduler/OleException", "GetComment failed with returncode %i", hr);
    return NULL;
  } else {
    jstring str = env->NewString(lpwszComment, wcslen(lpwszComment));
    CoTaskMemFree(lpwszComment);
    return str;
  }
}

JNIEXPORT void JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_SetComment(JNIEnv *env, jclass, jint task, jstring comment) {
  HRESULT hr = ERROR_SUCCESS;
  ITask *pITask = (ITask *)task;
  LPCWSTR lpwszComment = env->GetStringChars(comment, NULL);
  hr = pITask->SetComment(lpwszComment);
  if (FAILED(hr)) {
    JNU_ThrowByName(env, "de/steffensiebert/jwindowsscheduler/OleException", "SetComment failed with returncode %i", hr);
    return;
  }
}

JNIEXPORT jint JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_Enum(JNIEnv *env, jclass, jint taskScheduler) {
  HRESULT hr = ERROR_SUCCESS;
  IEnumWorkItems *pIEnum;
  ITaskScheduler *pITS = (ITaskScheduler *)taskScheduler;
  hr = pITS->Enum(&pIEnum);
  if (FAILED(hr)) {
    JNU_ThrowByName(env, "de/steffensiebert/jwindowsscheduler/OleException", "Enum failed with returncode %i", hr);
    return NULL;
  } else {
    return (jint) pIEnum;
  }
}
JNIEXPORT jstring JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_Next(JNIEnv *env, jclass, jint e) {
#define TASKS_TO_RETRIEVE 1
  HRESULT hr = ERROR_SUCCESS;
  IEnumWorkItems *pIEnum = (IEnumWorkItems *)e;
  LPWSTR *lpwszNames;
  DWORD dwFetchedTasks = 0;
  hr = pIEnum->Next(TASKS_TO_RETRIEVE,
                    &lpwszNames,
                    &dwFetchedTasks);
  if (FAILED(hr)) {
    JNU_ThrowByName(env, "de/steffensiebert/jwindowsscheduler/OleException", "Next failed with returncode %i", hr);
    return NULL;
  } else {
    if (dwFetchedTasks == 1) {
      jstring str = env->NewString(lpwszNames[0], wcslen(lpwszNames[0]));
      CoTaskMemFree(lpwszNames[0]);
      CoTaskMemFree(lpwszNames);
      return str;
    } else if (dwFetchedTasks == 0) {
      /* Keine weiteren Tasks */
      return NULL;
    } else {
      JNU_ThrowByName(env, "de/steffensiebert/jwindowsscheduler/OleException", "Next returned %i tasks instead of one", dwFetchedTasks);
      return NULL;
    }
  }
}

JNIEXPORT void JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_ReleaseEnum(JNIEnv *env, jclass, jint e) {
  IEnumWorkItems *pIEnum = (IEnumWorkItems *)e;
  pIEnum->Release();
}

JNIEXPORT jboolean JNICALL Java_de_steffensiebert_jwindowsscheduler_Ole_WaitForFileChange(JNIEnv *env, jclass, jstring dirName) {
  DWORD dwWaitStatus; 
  HANDLE dwChangeHandles[2]; 
  LPCWSTR lpwszDirName = env->GetStringChars(dirName, NULL);
 
  // Watch the directory for file writes

  dwChangeHandles[0] = FindFirstChangeNotificationW( 
                                                   lpwszDirName,                  // directory to watch 
                                                   FALSE,                          // do not watch the subtree 
                                                   FILE_NOTIFY_CHANGE_LAST_WRITE); // watch file name changes 
 
  if (dwChangeHandles[0] == INVALID_HANDLE_VALUE) {
    printf("Invalid Handle Value\n");
    return (jboolean) 0;
  } 
  // Change notification is set. Now wait on the notification 
  // handle
 
  while (TRUE) { 
 
    // Wait for notification.
 
    dwWaitStatus = WaitForMultipleObjects(1, dwChangeHandles, 
                                          FALSE, INFINITE); 
 
    switch (dwWaitStatus) { 
    case WAIT_OBJECT_0: 
      return (jboolean) 1;
      break; 
    default: 
      printf("default\n");
      return (jboolean) 0;
    } 
  }
}
